@@ -685,7 +685,7 @@ import inspect
 from distutils.version import StrictVersion
 
 # setup.py assumes the version is a simple string in '' quotes
-__version__ = '0.24'
+__version__ = '0.31'
 
 def _product(l):
     r'''Returns product of all values in the given list'''
@@ -887,7 +887,6 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
         >>> inner_product(a,b)
         array([ 305, 1250])
 
-
     The prototype defines the dimensionality of the inputs. In the inner product
     example above, the input is two 1D n-dimensional vectors. In particular, the
     'n' is the same for the two inputs. This function is intended to be used as
@@ -918,9 +917,9 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
         returns
       - The leading dimensions come from the extra dimensions in the inputs
 
-    Calling a function wrapped with broadcast_define() with more positional
-    arguments than specified in the prototype passes these extra positional
-    arguments verbatim to the inner function. Same with keyword arguments.
+    Calling a function wrapped with broadcast_define() with extra arguments
+    (either positional or keyword), passes these verbatim to the inner function.
+    Only the arguments declared in the prototype are broadcast.
 
     Scalars are represented as 0-dimensional numpy arrays: arrays with shape (),
     and these broadcast as one would expect:
@@ -1014,6 +1013,7 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
             .....
             out[:] = result
 
+    When used this way, the return value of the broadcasted function is ignored.
     In order for broadcast_define() to pass such an output array to the inner
     function, this output array must be available, which means that it must be
     given to us somehow, or we must create it.
@@ -1027,7 +1027,6 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
         def inner_product(a, b, out):
             .....
             out.setfield(a.dot(b), out.dtype)
-            return out
 
         out = np.empty((2,4), np.float)
         inner_product( np.arange(3), np.arange(2*4*3).reshape(2,4,3), out=out)
@@ -1052,7 +1051,6 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
         def inner_product(a, b, out):
             .....
             out.setfield(a.dot(b), out.dtype)
-            return out
 
         out = inner_product( np.arange(3), np.arange(2*4*3).reshape(2,4,3), dtype=int)
 
@@ -1083,12 +1081,12 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
         out = inner_product( np.arange(3), np.arange(2*4*3).reshape(2,4,3))
 
     Here we were slighly inefficient, but the ONLY required extra specification
-    was out_kwarg: that's mostly all you need. Also it is important to note that
-    in this case the inner function is called both with passing it an output
-    array to fill in, and with asking it to create a new one (by passing
-    out=None to the inner function). This inner function then must support both
-    modes of operation. If the inner function does not support filling in an
-    output array, none of these efficiency improvements are possible.
+    was out_kwarg: that's all you need. Also it is important to note that in
+    this case the inner function is called both with passing it an output array
+    to fill in, and with asking it to create a new one (by passing out=None to
+    the inner function). This inner function then must support both modes of
+    operation. If the inner function does not support filling in an output
+    array, none of these efficiency improvements are possible.
 
     It is possible for a function to return more than one output, and this is
     supported by broadcast_define(). This case works exactly like the one-output
@@ -1150,20 +1148,8 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
             if Noutputs is None:
                 # We expect a SINGLE output
 
-                # if no broadcasting involved, just call the function
-                if not dims_extra:
-                    sliced_args = args + args_passthru
-                    result = func( *sliced_args, **kwargs )
-                    if isinstance(result, tuple):
-                        raise NumpysaneError("Only a single output expected, but a tuple was returned!")
-                    if prototype_output_expanded is not None and \
-                       np.array(result).shape != tuple(prototype_output_expanded):
-                        raise NumpysaneError("Inconsistent slice output shape: expected {}, but got {}".format(prototype_output_expanded,
-                                                                                                               np.array(result).shape))
-                    return result
-
                 # if the output was supposed to go to a particular place, set that
-                if out_kwarg and out_kwarg in kwargs:
+                if out_kwarg is not None and out_kwarg in kwargs:
                     output = kwargs[out_kwarg]
                     if prototype_output_expanded is not None:
                         expected_shape = dims_extra + prototype_output_expanded
@@ -1176,6 +1162,36 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
                         kwargs_dtype['dtype'] = kwargs['dtype']
                     output = np.empty(dims_extra + prototype_output_expanded,
                                       **kwargs_dtype)
+                # else:
+                #   We don't have an output and we don't know its dimensions, so
+                #   we can't allocate an array for it. Leave output as None. I
+                #   will allocate it as soon I get the first slice; this will let
+                #   me know how large the whole thing should be
+
+                # if no broadcasting involved, just call the function
+                if not dims_extra:
+                    # if the function knows how to write directly to an array,
+                    # request that
+                    if output is not None and out_kwarg is not None:
+                        kwargs[out_kwarg] = output
+
+                    sliced_args = args + args_passthru
+                    result = func( *sliced_args, **kwargs )
+
+                    if out_kwarg             is not None and \
+                       kwargs.get(out_kwarg) is not None:
+                        # We wrote the output in-place. Return the output array
+                        return kwargs.get(out_kwarg)
+
+                    # Using the returned output. Run some checks, and return the
+                    # returned value
+                    if isinstance(result, tuple):
+                        raise NumpysaneError("Only a single output expected, but a tuple was returned!")
+                    if prototype_output_expanded is not None and \
+                       np.array(result).shape != tuple(prototype_output_expanded):
+                        raise NumpysaneError("Inconsistent slice output shape: expected {}, but got {}".format(prototype_output_expanded,
+                                                                                                               np.array(result).shape))
+                    return result
 
                 # reshaped output. I write to this array
                 if output is not None:
@@ -1190,35 +1206,77 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
 
                     sliced_args = x + args_passthru
                     result = func( *sliced_args, **kwargs )
-                    if isinstance(result, tuple):
-                        raise NumpysaneError("Only a single output expected, but a tuple was returned!")
 
-                    if not isinstance(result, np.ndarray):
-                        result = np.array(result)
+                    if output is None or out_kwarg is None:
+                        # We weren't writing directly into the output, so check
+                        # the output for validity
+                        if isinstance(result, tuple):
+                            raise NumpysaneError("Only a single output expected, but a tuple was returned!")
+                        if not isinstance(result, np.ndarray):
+                            result = np.array(result)
+
+                        if prototype_output_expanded is None:
+                            prototype_output_expanded = result.shape
+                        else:
+                            if result.shape != tuple(prototype_output_expanded):
+                                raise NumpysaneError("Inconsistent slice output shape: expected {}, but got {}".format(prototype_output_expanded, result.shape))
 
                     if output is None:
+                        # I didn't already have an output array because I didn't
+                        # know how large it should be. But I now have the first
+                        # slice, and I know how big the whole output should be.
+                        # I create it
                         output = np.empty( dims_extra + list(result.shape),
                                            dtype = result.dtype)
                         output_flattened = output.reshape( (_product(dims_extra),) + result.shape)
                         output_flattened[i_slice, ...] = result
 
-                    elif not out_kwarg:
+                    elif out_kwarg is None:
                         output_flattened[i_slice, ...] = result
+                    # else:
+                    #   I was writing directly to the output, so I don't need to
+                    #   manually populate the slice
 
-                    if prototype_output_expanded is None:
-                        prototype_output_expanded = result.shape
-                    else:
-                        if result.shape != tuple(prototype_output_expanded):
-                            raise NumpysaneError("Inconsistent slice output shape: expected {}, but got {}".format(prototype_output_expanded, result.shape))
                     i_slice = i_slice+1
 
             else:
                 # We expect MULTIPLE outputs: a tuple of length Noutputs
 
+                # if the output was supposed to go to a particular place, set that
+                if out_kwarg is not None and out_kwarg in kwargs:
+                    output = kwargs[out_kwarg]
+                    if prototype_output_expanded is not None:
+                        for i in range(Noutputs):
+                            expected_shape = dims_extra + prototype_output_expanded[i]
+                            if output[i].shape != tuple(expected_shape):
+                                raise NumpysaneError("Inconsistent output shape for output {}: expected {}, but got {}". \
+                                                     format(i, expected_shape, output[i].shape))
+                # if we know enough to allocate the output, do that
+                elif prototype_output_expanded is not None:
+                    kwargs_dtype = {}
+                    if 'dtype' in kwargs:
+                        kwargs_dtype['dtype'] = kwargs['dtype']
+                    output = [np.empty(dims_extra + prototype_output_expanded[i],
+                                       **kwargs_dtype) for i in range(Noutputs)]
+                # else:
+                #   We don't have an output and we don't know its dimensions, so
+                #   we can't allocate an array for it. Leave output as None. I
+                #   will allocate it as soon I get the first slice; this will let
+                #   me know how large the whole thing should be
+
                 # if no broadcasting involved, just call the function
                 if not dims_extra:
+                    # if the function knows how to write directly to an array,
+                    # request that
+                    if output is not None and out_kwarg is not None:
+                        kwargs[out_kwarg] = tuple(output)
+
                     sliced_args = args + args_passthru
                     result = func( *sliced_args, **kwargs )
+                    if out_kwarg             is not None and \
+                       kwargs.get(out_kwarg) is not None:
+                        # We wrote the output in-place. Return the output array
+                        return kwargs.get(out_kwarg)
 
                     if not isinstance(result, tuple):
                         raise NumpysaneError("A tuple of {} outputs is expected, but an object of type {} was returned". \
@@ -1236,23 +1294,6 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
                     return result
 
 
-                # if the output was supposed to go to a particular place, set that
-                if out_kwarg and out_kwarg in kwargs:
-                    output = kwargs[out_kwarg]
-                    if prototype_output_expanded is not None:
-                        for i in range(Noutputs):
-                            expected_shape = dims_extra + prototype_output_expanded[i]
-                            if output[i].shape != tuple(expected_shape):
-                                raise NumpysaneError("Inconsistent output shape for output {}: expected {}, but got {}". \
-                                                     format(i, expected_shape, output[i].shape))
-                # if we know enough to allocate the output, do that
-                elif prototype_output_expanded is not None:
-                    kwargs_dtype = {}
-                    if 'dtype' in kwargs:
-                        kwargs_dtype['dtype'] = kwargs['dtype']
-                    output = [np.empty(dims_extra + prototype_output_expanded[i],
-                                       **kwargs_dtype) for i in range(Noutputs)]
-
                 # reshaped output. I write to this array
                 if output is not None:
                     output_flattened = [clump(output[i], n=len(dims_extra)) for i in range(Noutputs)]
@@ -1266,33 +1307,45 @@ def broadcast_define(prototype, prototype_output=None, out_kwarg=None):
 
                     sliced_args = x + args_passthru
                     result = func( *sliced_args, **kwargs )
-                    if not isinstance(result, tuple):
-                        raise NumpysaneError("A tuple of {} outputs is expected, but an object of type {} was returned". \
-                                             format(Noutputs, type(result)))
-                    if len(result) != Noutputs:
-                        raise NumpysaneError("A tuple of {} outputs is expected, but a length-{} tuple was returned". \
-                                             format(Noutputs, len(result)))
 
-                    result = [x if isinstance(x, np.ndarray) else np.array(x) for x in result]
+                    if output is None or out_kwarg is None:
+                        # We weren't writing directly into the output, so check
+                        # the output for validity
+                        if not isinstance(result, tuple):
+                            raise NumpysaneError("A tuple of {} outputs is expected, but an object of type {} was returned". \
+                                                 format(Noutputs, type(result)))
+                        if len(result) != Noutputs:
+                            raise NumpysaneError("A tuple of {} outputs is expected, but a length-{} tuple was returned". \
+                                                 format(Noutputs, len(result)))
+
+                        result = [x if isinstance(x, np.ndarray) else np.array(x) for x in result]
+
+                        if prototype_output_expanded is None:
+                            prototype_output_expanded = [result[i].shape for i in range(Noutputs)]
+                        else:
+                            for i in range(Noutputs):
+                                if result[i].shape != tuple(prototype_output_expanded[i]):
+                                    raise NumpysaneError("Inconsistent slice output {} shape: expected {}, but got {}". \
+                                                         format(i, prototype_output_expanded[i], result[i].shape))
 
                     if output is None:
+                        # I didn't already have an output array because I didn't
+                        # know how large it should be. But I now have the first
+                        # slice, and I know how big the whole output should be.
+                        # I create it
                         output = [np.empty( dims_extra + list(result[i].shape),
                                             dtype = result[i].dtype) for i in range(Noutputs)]
                         output_flattened = [output[i].reshape( (_product(dims_extra),) + result[i].shape) for i in range(Noutputs)]
                         for i in range(Noutputs):
                             output_flattened[i][i_slice, ...] = result[i]
 
-                    elif not out_kwarg:
+                    elif out_kwarg is None:
                         for i in range(Noutputs):
                             output_flattened[i][i_slice, ...] = result[i]
+                    # else:
+                    #   I was writing directly to the output, so I don't need to
+                    #   manually populate the slice
 
-                    if prototype_output_expanded is None:
-                        prototype_output_expanded = [result[i].shape for i in range(Noutputs)]
-                    else:
-                        for i in range(Noutputs):
-                            if result[i].shape != tuple(prototype_output_expanded[i]):
-                                raise NumpysaneError("Inconsistent slice output {} shape: expected {}, but got {}". \
-                                                     format(i, prototype_output_expanded[i], result[i].shape))
                     i_slice = i_slice+1
 
             return output
@@ -1572,10 +1625,17 @@ def glue(*args, **kwargs):
     # deal with scalar (non-ndarray) args
     args = [ np.asarray(x) for x in args ]
 
-    # ignore empty arrays( shape == (0,) ) but not scalars ( shape == ())
-    args = [ x for x in args if x.shape != (0,) ]
-    if not args:
-        return np.zeros((0,))
+    # Special case to support this common idiom:
+    #
+    #   accum = np.array(())
+    #   while ...:
+    #      x     = ...
+    #      accum = nps.glue(accum, x,  axis = -2)
+    #
+    # Without special logic, this would throw an error since accum.shape starts
+    # at (0,), which is almost certainly not compatible with x.shape
+    if len(args) == 2 and args[0].shape == (0,) and args[1].size != 0:
+        return args[1]
 
     # Legacy behavior: if no axis is given, add a new axis at the front, and
     # glue along it
@@ -1939,8 +1999,9 @@ def transpose(x):
     '''
     return xchg( atleast_dims(x, -2), -1, -2)
 
-def dummy(x, axis=None):
-    r'''Adds a single length-1 dimension at the given position.
+
+def dummy(x, axis, *axes_rest):
+    r'''Adds length-1 dimensions at the given positions.
 
     SYNOPSIS
 
@@ -1963,6 +2024,9 @@ def dummy(x, axis=None):
         >>> nps.dummy(a, -2).shape
         (2, 3, 1, 4)
 
+        >>> nps.dummy(a, -2, -2).shape
+        (2, 3, 1, 1, 4)
+
         >>> nps.dummy(a, -5).shape
         (1, 1, 2, 3, 4)
 
@@ -1972,14 +2036,21 @@ def dummy(x, axis=None):
     are added.
 
     '''
-    need_ndim = axis+1 if axis >= 0 else -axis
-    if x.ndim >= need_ndim:
-        # referring to an axis that already exists. expand_dims() thus works
-        return np.expand_dims(x, axis)
 
-    # referring to a non-existing axis. I simply add sufficient new axes, and
-    # I'm done
-    return atleast_dims(x, axis)
+    def dummy_inner(x, axis):
+        need_ndim = axis+1 if axis >= 0 else -axis
+        if x.ndim >= need_ndim:
+            # referring to an axis that already exists. expand_dims() thus works
+            return np.expand_dims(x, axis)
+
+        # referring to a non-existing axis. I simply add sufficient new axes, and
+        # I'm done
+        return atleast_dims(x, axis)
+
+    axes = (axis,) + axes_rest
+    for axis in axes: x = dummy_inner(x, axis)
+    return x
+
 
 def reorder(x, *dims):
     r'''Reorders the dimensions of an array.
@@ -2038,16 +2109,25 @@ def dot(a, b, out=None, dtype=None):
         >>> nps.dot(a,b)
         20
 
-    this is identical to numpysane.inner(). for a conjugating version of this
-    function, use nps.vdot(). note that the numpy dot() has some special
-    handling when its dot() is given more than 1-dimensional input. this
-    function has no special handling: normal broadcasting rules are applied.
+    This is identical to numpysane.inner(). for a conjugating version of this
+    function, use nps.vdot(). Note that the stock numpy dot() has some special
+    handling when its dot() is given more than 1-dimensional input. THIS
+    function has no special handling: normal broadcasting rules are applied, as
+    expected.
+
+    In-place operation is available with the "out" kwarg. The output dtype can
+    be selected with the "dtype" kwarg. If omitted, the dtype of the input is
+    used
 
     '''
     if out is not None and dtype is not None and out.dtype != dtype:
         raise NumpysaneError("'out' and 'dtype' given explicitly, but the dtypes are mismatched!")
 
-    v = np.sum(a*b, axis=-1, out=out, dtype=dtype )
+    if dtype is not None:
+        # Handle overflows. Cases that require this are checked in the tests
+        v = np.sum(a.astype(dtype)*b.astype(dtype), axis=-1, out=out, dtype=dtype )
+    else:
+        v = np.sum(a*b, axis=-1, out=out, dtype=dtype )
     if out is None:
         return v
     return out
@@ -2154,7 +2234,7 @@ def norm2(a, **kwargs):
     '''
     return inner(a,a, **kwargs)
 
-def mag(a, out=None):
+def mag(a, out=None, dtype=None):
     r'''Magnitude of a vector. mag(x) is functionally identical to sqrt(inner(x,x))
 
     SYNOPSIS
@@ -2170,13 +2250,17 @@ def mag(a, out=None):
         2.23606797749979
 
     This is a convenience function to compute a magnitude of a vector, with full
-    broadcasting support. If and explicit "out" array isn't given, we produce
-    output of dtype=float. Otherwise "out" retains its dtype
+    broadcasting support.
+
+    In-place operation is available with the "out" kwarg. The output dtype can
+    be selected with the "dtype" kwarg. If omitted, dtype=float is selected.
 
     '''
 
     if out is None:
-        out = inner(a,a, dtype=float)
+        if dtype is None:
+            dtype = float
+        out = inner(a,a, dtype=dtype)
 
         if not isinstance(out, np.ndarray):
             # given two vectors, and without and 'out' array, inner() produces a
@@ -2184,7 +2268,9 @@ def mag(a, out=None):
             # return a copy
             return np.sqrt(out)
     else:
-        inner(a,a, out=out)
+        if dtype is None:
+            dtype = out.dtype
+        inner(a,a, out=out, dtype=dtype)
 
     # in-place sqrt
     np.sqrt.at(out,())
