@@ -436,7 +436,7 @@ validation code, you can just return false, and a generic Exception will be
 thrown. Or you can throw a more informative exception yourself prior to
 returning false.
 
-** Arguments available to the code snippets
+** Values available to the code snippets
 Each of the user-supplied code blocks is placed into a separate function in the
 generated code, with identical arguments in both cases. These arguments describe
 the inputs and outputs, and are meant to be used by the user code. We have
@@ -469,7 +469,7 @@ strides contain the step size in bytes, of each dimension. sizeof_element...
 describes the size in bytes, of a single data element.
 
 Finally, I have a pointer to the data itself. The validation code gets a pointer
-to the start of the data array:
+to the start of the whole data array:
 
     void*           data__NAME
 
@@ -478,19 +478,43 @@ currently looking at:
 
     void*           data_slice__NAME
 
+If the data in the arrays is representable as a basic C type (most integers,
+floats and complex numbers), then convenience macros are available to index
+elements in the sliced arrays and to conveniently access the C type of the data.
+These macros take into account the data type and the strides.
+
+    #define         ctype__NAME     ...
+    #define         item__NAME(...) ...
+
+For instance, if we have a 2D array 'x' containing 64-bit floats, we'll have
+this:
+
+    #define         ctype__x     npy_float64 /* "double" on most platforms */
+    #define         item__x(i,j) (*(ctype__x*)(data_slice__x + ...))
+
+For more complex types (objects, vectors, strings) you'll need to deal with the
+strides and the pointers yourself.
+
 Example: I'm computing a broadcasted slice. An input array 'x' is a
 2-dimensional slice of dimension (3,4) of 64-bit floating-point values. I thus
 have Ndims_slice__x == 2 and dims_slice__x[] = {3,4} and sizeof_element__x == 8.
-An element of this array at i,j is
+An element of this array at i,j can be accessed with either
 
     *((double*)(data_slice__a + i*strides_slice__a[0] + j*strides_slice__a[1]))
 
-If I defined a validation function that makes sure that 'a' is stored in
-contiguous memory, the computation code doesn't need to look at the strides at
-all, and element at i,j can be found more simply:
+    item__a(i,j)
+
+Both are identical. If I defined a validation function that makes sure that 'a'
+is stored in contiguous memory, the computation code doesn't need to look at the
+strides at all, and element at i,j can be found more simply:
 
     ((double*)data_slice__a)[ i*dims_slice__a[1] + j ]
 
+    item__a(i,j)
+
+As you can see, the item__...() macros are much simpler, less error-prone and
+are thus the preferred form.
+
 ** Specifying extra, non-broadcasted arguments
 
 Sometimes it is desired to pass extra arguments to the C code; ones that aren't
@@ -691,10 +715,22 @@ import re
 # and to appear in a "normal" directory, where this script can grab them and use
 # them
 #
-# Aand I try two different directories, in case I'm running in-place
+# And I try two different directories, in case I'm running in-place
+#
+# And pip does something yet different, which I support in a hacky way. This is
+# a mess
 
-_pywrap_path = ( os.path.dirname( __file__ ) + '/pywrap-templates',
-                 sys.prefix + '/share/python-numpysane/pywrap-templates' )
+_pywrap_path = [ # in-place: running from the source tree
+                 os.path.dirname( __file__ ) + '/pywrap-templates',
+
+                 # distro: /usr/share/...
+                 sys.prefix + '/share/python-numpysane/pywrap-templates' ]
+
+                 # pip: /home/whoever/.local/share/...
+_m = re.match("(/home/[^/]+/\.local)/lib/", __file__)
+if _m is not None:
+    _local_prefix = _m.group(1)
+    _pywrap_path.append( _local_prefix + '/share/python-numpysane/pywrap-templates')
 
 for p in _pywrap_path:
     _module_header_filename = p + '/pywrap_module_header.c'
@@ -720,24 +756,17 @@ def _quote(s, convert_newlines=False):
     return s
 
 
-def _substitute(s, convert_newlines=False, **kwargs):
+def _substitute(s, **kwargs):
     r'''format() with specific semantics
 
-    - {xxx} substitutions founc in kwargs are made
+    - {xxx} substitutions found in kwargs are made
     - {xxx} expressions not found in kwargs are left as is
     - {{ }} escaping is not respected: any '{xxx}' is replaced
-    - \ and " and \n are handled for C strings
-    - if convert_newlines: newlines are converted to \n (useful for C strings).
       Otherwise they're left alone (useful for C code)
 
     '''
 
     for k in kwargs.keys():
-        v = kwargs[k]
-        if isinstance(v, str):
-            v = _quote(v, convert_newlines)
-        else:
-            v = str(v)
         s = s.replace('{' + k + '}', kwargs[k])
     return s
 
@@ -776,9 +805,7 @@ class module:
         with open( _module_footer_filename, 'r') as f:
             self.module_footer = _substitute(f.read(),
                                              MODULE_NAME      = name,
-                                             MODULE_DOCSTRING = docstring,
-                                             convert_newlines = True)
-
+                                             MODULE_DOCSTRING = _quote(docstring, convert_newlines=True))
         self.functions = []
 
 
@@ -1107,8 +1134,6 @@ class module:
 '''.replace('{Noutputs}', str(Noutputs))
 
 
-        Ntypesets = len(Ccode_slice_eval)
-        slice_functions = [ "__{}__{}__slice".format(name,i) for i in range(Ntypesets)]
         # The keys of Ccode_slice_eval are either:
 
         # - a type: all inputs, outputs MUST have this type
@@ -1128,7 +1153,7 @@ class module:
                 # already a list of types. we're good
                 pass
             else:
-                raise NumpysaneError("Each of Ccode_slice_eval.keys() MUST be either a type, or a list of types (one per input, output in order)")
+                raise NumpysaneError("Each of Ccode_slice_eval.keys() MUST be either a type, or a list of types (one for each input followed by one for each output in order; {} + {} = {} total)".format(Ninputs, Ninputs_and_outputs-Ninputs, Ninputs_and_outputs))
 
         # {TYPESETS} is _(11, 15, 17, 0) _(13, 15, 17, 1)
         # {TYPESET_MATCHES_ARGLIST} is t0,t1,t2
@@ -1136,7 +1161,7 @@ class module:
         #                         "   (float32,int32)\n"
         #                         "   (float64,int32)\n"
         TYPESETS = ' '.join( ("_(" + ','.join(tuple(str(np.dtype(t).num) for t in known_types[i]) + (str(i),)) + ')') \
-                              for i in range(Ntypesets))
+                              for i in range(len(known_types)))
         TYPESET_MATCHES_ARGLIST = ','.join(('t' + str(i)) for i in range(Ninputs_and_outputs))
         def parened_type_list(l, Ninputs):
             r'''Converts list of types to string
@@ -1175,9 +1200,12 @@ bool {FUNCTION_NAME}({ARGUMENTS})
 }
 '''
         if Noutputs is None:
-            slice_args = ("output",)+args_input
+            slice_args = ("output",) + args_input
         else:
             slice_args = tuple("output{}".format(i) for i in range(Noutputs))+args_input
+        slice_args_ndims = \
+            [ len(prototype) for prototype in prototype_outputs ] + \
+            [ len(prototype) for prototype in prototype_input   ]
 
         EXTRA_ARGUMENTS_ARG_DEFINE          = ''
         EXTRA_ARGUMENTS_NAMELIST            = ''
@@ -1224,25 +1252,56 @@ bool {FUNCTION_NAME}({ARGUMENTS})
         EXTRA_ARGUMENTS_ARGLIST_PARSE_PYARG = ''.join([s+',' for s in EXTRA_ARGUMENTS_ARGLIST_PARSE_PYARG])
         EXTRA_ARGUMENTS_ARGLIST_CALL_C      = ','.join(EXTRA_ARGUMENTS_ARGLIST_CALL_C)
 
+        def ctype_from_dtype(t):
+            f'''Get the corresponding C type from a numpy dtype
+
+If one does not exist, return None'''
+            t = np.dtype(t)
+            if not t.isnative or not (t.isbuiltin==1): return None
+
+            nbits = str(t.itemsize * 8)
+            if t.kind == 'f': return "npy_float"   + nbits
+            if t.kind == 'c': return "npy_complex" + nbits
+            if t.kind == 'i': return "npy_int"     + nbits
+            if t.kind == 'u': return "npy_uint"    + nbits
+            return None
+
+
+
         text = ''
         contiguous_macro_template = r'''
-#define _CHECK_CONTIGUOUS__{name}(seterror)                                       \
-({                                                                                \
-  bool result = true;                                                             \
-  int Nelems_slice = 1;                                                           \
-  for(int i=-1; i>=-Ndims_slice__{name}; i--)                                     \
-  {                                                                               \
-      if(strides_slice__{name}[i+Ndims_slice__{name}] != sizeof_element__{name}*Nelems_slice) \
-      {                                                                           \
-          result = false;                                                         \
-          if(seterror)                                                            \
-            PyErr_Format(PyExc_RuntimeError,                                      \
-                         "Variable '{name}' must be contiguous in memory, and it isn't in (at least) dimension %d", i); \
-          break;                                                                  \
-      }                                                                           \
-      Nelems_slice *= dims_slice__{name}[i+Ndims_slice__{name}];                  \
-  }                                                                               \
-  result;                                                                         \
+#define _CHECK_CONTIGUOUS__{name}(seterror)                             \
+({                                                                      \
+  bool result     = true;                                               \
+  bool have_dim_0 = false;                                              \
+  /* If I have no data, just call the thing contiguous. This is useful */ \
+  /* because np.ascontiguousarray doesn't set contiguous alignment */   \
+  /* for empty arrays */                                                \
+  for(int i=0; i<Ndims_full__{name}; i++)                               \
+      if(dims_full__{name}[i] == 0)                                     \
+      {                                                                 \
+          result     = true;                                            \
+          have_dim_0 = true;                                            \
+          break;                                                        \
+      }                                                                 \
+                                                                        \
+  if(!have_dim_0)                                                       \
+  {                                                                     \
+      int Nelems_slice = 1;                                             \
+      for(int i=-1; i>=-Ndims_slice__{name}; i--)                       \
+      {                                                                 \
+          if(strides_slice__{name}[i+Ndims_slice__{name}] != sizeof_element__{name}*Nelems_slice) \
+          {                                                             \
+              result = false;                                           \
+              if(seterror)                                              \
+                PyErr_Format(PyExc_RuntimeError,                        \
+                             "Variable '{name}' must be contiguous in memory, and it isn't in (at least) dimension %d", i); \
+              break;                                                    \
+          }                                                             \
+          Nelems_slice *= dims_slice__{name}[i+Ndims_slice__{name}];    \
+      }                                                                 \
+  }                                                                     \
+  result;                                                               \
 })
 #define CHECK_CONTIGUOUS__{name}()              _CHECK_CONTIGUOUS__{name}(false)
 #define CHECK_CONTIGUOUS_AND_SETERROR__{name}() _CHECK_CONTIGUOUS__{name}(true)
@@ -1294,14 +1353,44 @@ typedef struct { {COOKIE_STRUCT_CONTENTS} } __{FUNCTION_NAME}__cookie_t;
                         ARGUMENTS     = _substitute(arglist_string, DATA_ARGNAME="data"),
                         FUNCTION_BODY = "return true;" if Ccode_validate is None else Ccode_validate)
 
-        for i in range(Ntypesets):
-            # The evaluation function for one slice
-            typeset_indices = tuple(Ccode_slice_eval.keys())
+        # The evaluation function for one slice
+        known_typesets = list(Ccode_slice_eval.keys()) # known_types is the same, but tweaked
+        for i_typeset in range(len(known_typesets)):
+
+            slice_function = "__{}__{}__slice".format(name,i_typeset)
+
+            text += '\n'
+            text_undef = ''
+            for i_arg in range(len(slice_args)):
+                ctype = ctype_from_dtype(known_types[i_typeset][i_arg])
+                if ctype is None:
+                    continue
+
+                arg_name  = slice_args      [i_arg]
+                ndims     = slice_args_ndims[i_arg]
+                text_here = \
+                    '#define ctype__{name} {ctype}\n' +                                 \
+                    '#define item__{name}(' +                                           \
+                    ','.join([ "__ivar"+str(i) for i in range(ndims)]) +                \
+                    ') (*(ctype__{name}*)(data_slice__{name} ' +                        \
+                    ''.join(['+ (__ivar' + str(i) + ')*strides_slice__{name}['+str(i)+']' \
+                             for i in range(ndims)]) +                                  \
+                    '))\n'
+
+                text += _substitute(text_here,
+                                    name = arg_name,
+                                    ctype= ctype)
+                text_undef += '#undef item__{name}\n' .replace('{name}', arg_name)
+                text_undef += '#undef ctype__{name}\n'.replace('{name}', arg_name)
+
             text += \
                 _substitute(function_template,
-                            FUNCTION_NAME = slice_functions[i],
+                            FUNCTION_NAME = slice_function,
                             ARGUMENTS     = _substitute(arglist_string, DATA_ARGNAME="data_slice"),
-                            FUNCTION_BODY = Ccode_slice_eval[typeset_indices[i]])
+                            FUNCTION_BODY = Ccode_slice_eval[known_typesets[i_typeset]])
+
+            text += text_undef
+
 
         text += \
             ' \\\n  '.join(ARGUMENTS_LIST) + \
@@ -1327,6 +1416,8 @@ typedef struct { {COOKIE_STRUCT_CONTENTS} } __{FUNCTION_NAME}__cookie_t;
             text += '#undef _CHECK_CONTIGUOUS__{name}\n'.replace('{name}', n)
             text += '#undef CHECK_CONTIGUOUS__{name}\n'.replace('{name}', n)
             text += '#undef CHECK_CONTIGUOUS_AND_SETERROR__{name}\n'.replace('{name}', n)
+
+        text += '\n'
         text += '#undef CHECK_CONTIGUOUS_ALL\n'
         text += '#undef CHECK_CONTIGUOUS_AND_SETERROR_ALL\n'
 
