@@ -364,16 +364,21 @@ def test_broadcasting():
     confirm( f13(arr(5), arr(2,5)) is not None, msg='output count check' )
 
     # check output dimensionality with an 'out' kwarg
+    @nps.broadcast_define( (('n',), ('n',)), ('n', 'n'),
+                           out_kwarg = 'out')
+    def f14_oneoutput(a, b, out):
+        if out is None:
+            return nps.outer(a,b)
+        nps.outer(a,b,out=out)
     @nps.broadcast_define( (('n',), ('n',)), ((),('n', 'n')),
                            out_kwarg = 'out')
-    def f14(a, b, out=None):
+    def f14(a, b, out):
         if out is None:
             return a.dot(b),nps.outer(a,b)
         if not isinstance(out,tuple) or len(out) != 2:
             raise Exception("'out' must be a tuple")
         nps.inner(a,b,out=out[0])
         nps.outer(a,b,out=out[1])
-        return out
 
     confirm( f14(arr(5), arr(  5)) is not None,
              msg='output dimensionality check with out_kwarg' )
@@ -409,7 +414,12 @@ def test_broadcasting():
                                 msg='output dimensionality check with out_kwarg' )
     confirm_raises( lambda: f14(a5, a5, out=(o,o55,o)),
                                 msg='output dimensionality check with out_kwarg' )
-    confirm( f14(a5, a5, out=(o,o55)) is not None,
+    confirm_does_not_raise( lambda: f14_oneoutput(a5, a5, out=o55),
+             msg='output dimensionality check with out_kwarg' )
+    confirm_equal(o55, np.outer(a5,a5), msg='in-place broadcasting computed the right value')
+    confirm_equal( o55, f14_oneoutput(a5, a5),
+                   msg='in-place broadcasting computed the right value')
+    confirm_does_not_raise( lambda: f14(a5, a5, out=(o,o55)),
              msg='output dimensionality check with out_kwarg' )
     confirm_equal(o,   a5.dot(a5),      msg='in-place broadcasting computed the right value')
     confirm_equal(o55, np.outer(a5,a5), msg='in-place broadcasting computed the right value')
@@ -439,10 +449,13 @@ def test_broadcasting():
                                 msg='output dimensionality check with out_kwarg' )
     confirm_raises( lambda: f14(a5, a25, out=(o,o255)),
                                 msg='output dimensionality check with out_kwarg' )
-    confirm( f14(a5, a25, out=(o2,o255)) is not None,
+    confirm_does_not_raise( lambda: f14(a5, a25, out=(o2,o255)),
              msg='output dimensionality check with out_kwarg' )
     confirm_equal(o2,   nps.inner(a5,a25), msg='in-place broadcasting computed the right value')
     confirm_equal(o255, nps.outer(a5,a25), msg='in-place broadcasting computed the right value')
+    (o2,o255) = f14(a5, a25)
+    confirm_equal(o2,   nps.inner(a5,a25), msg='in-place broadcasting computed the right value')
+    confirm_equal(o255, nps.outer(a5,a25), msg='in-place broadcasting computed the right value')
 
 
 def test_broadcasting_into_output():
@@ -778,12 +791,8 @@ def test_concatenation():
     confirm_raises( lambda: nps.glue( arr(1,3), arr(2,3), axis=-4) )
     confirm_raises( lambda: nps.cat(  arr(1,3), arr(2,3)) )
 
-    # empty arrays are accepted and ignored
-    assertValueShape( None, (2,3),     nps.glue, np.array(()), arr(2,3),     axis=-2 )
-    assertValueShape( None, (1,2,3),   nps.glue, np.array(()), arr(2,3),     axis=-3 )
     assertValueShape( None, (3,),      nps.glue, arr(3),       np.array(()), axis=-1 )
-    assertValueShape( None, (1,3),     nps.glue, arr(3),       np.array(()), axis=-2 )
-    assertValueShape( None, (3,3),     nps.glue, arr(3),       arr(2,3), np.array(()), axis=-2 )
+    assertValueShape( None, (4,),      nps.glue, arr(3),       np.array(5),  axis=-1 )
 
     # zero-length arrays do the right thing
     confirm_raises( lambda: nps.glue( arr(0,3), arr(2,3),     axis=-1) )
@@ -812,19 +821,23 @@ def test_concatenation():
     confirm_raises( lambda: nps.glue( arr(2,0), arr(3,0),     axis=-1) )
     assertValueShape( None, (5,0),     nps.glue, arr(2,0), arr(3,0),     axis=-2 )
     confirm_raises( lambda: nps.glue( arr(2,0), arr(3,0),     axis=-3) )
+    assertValueShape( None, (2,0),     nps.glue, arr(1,0), arr(1,0),     axis=-2 )
+    assertValueShape( None, (2,0),     nps.glue, arr(0,),  arr(1,0),     axis=-2 )
+    assertValueShape( None, (2,0),     nps.glue, arr(1,0), arr(0,),      axis=-2 )
+    assertValueShape( None, (0,),      nps.glue, arr(0,),  arr(0,),      axis=-1 )
+    assertValueShape( None, (2,0),     nps.glue, arr(0,),  arr(0,),      axis=-2 )
+    assertValueShape( None, (2,1,1,0), nps.glue, arr(0,),  arr(0,),      axis=-4 )
 
     assertValueShape( None, (0,),      nps.glue, arr(0,),  arr(0,),      axis=-1 )
     assertValueShape( None, (2,),      nps.glue, arr(2,),  arr(0,),      axis=-1 )
     assertValueShape( None, (2,),      nps.glue, arr(0,),  arr(2,),      axis=-1 )
-    assertValueShape( None, (1,2,),    nps.glue, arr(2,),  arr(0,),      axis=-2 )
-    assertValueShape( None, (1,2,),    nps.glue, arr(0,),  arr(2,),      axis=-2 )
+    confirm_raises(   lambda: nps.glue( arr(2,),  arr(0,), axis=-2 ) )
+    assertValueShape( None, (2,),      nps.glue, arr(0,),  arr(2,),      axis=-2 )
 
     # same as before, but np.array(()) instead of np.arange(0)
     assertValueShape( None, (0,),      nps.glue, np.array(()), np.array(()), axis=-1 )
     assertValueShape( None, (2,),      nps.glue, arr(2,),  np.array(()), axis=-1 )
     assertValueShape( None, (2,),      nps.glue, np.array(()),arr(2,),   axis=-1 )
-    assertValueShape( None, (1,2,),    nps.glue, arr(2,),  np.array(()), axis=-2 )
-    assertValueShape( None, (1,2,),    nps.glue, np.array(()),  arr(2,), axis=-2 )
 
     assertValueShape( None, (0,6),     nps.glue, arr(0,3), arr(0,3),     axis=-1 )
     assertValueShape( None, (0,3),     nps.glue, arr(0,3), arr(0,3),     axis=-2 )
@@ -913,6 +926,7 @@ def test_dimension_manipulation():
     assertValueShape( None, (2,1,3,4),   nps.dummy,        arr(2,3,4),  1 )
     assertValueShape( None, (2,3,4,1),   nps.dummy,        arr(2,3,4), -1 )
     assertValueShape( None, (2,3,1,4),   nps.dummy,        arr(2,3,4), -2 )
+    assertValueShape( None, (2,3,1,1,4), nps.dummy,        arr(2,3,4), -2, -2 )
     assertValueShape( None, (2,1,3,4),   nps.dummy,        arr(2,3,4), -3 )
     assertValueShape( None, (1,2,3,4),   nps.dummy,        arr(2,3,4), -4 )
     assertValueShape( None, (1,1,2,3,4), nps.dummy,        arr(2,3,4), -5 )
@@ -1006,7 +1020,7 @@ def test_inner():
 
 
 def test_mag():
-    r'''Testing the broadcasted magnitude product'''
+    r'''Testing the broadcasted norm2, magnitude'''
 
     # input is a 1D array of integers, no output dtype specified. Output
     # should be a floating-point scalar
@@ -1018,6 +1032,13 @@ def test_mag():
     assertResult_inoutplace(  np.sqrt(nps.norm2(np.arange(5))),
                               nps.mag, arr(5, dtype=float) )
 
+    # input is a 1D array of integers, no output dtype specified. Output
+    # should be a floating-point scalar
+    assertResult_inoutplace(  np.sqrt(nps.norm2(np.arange(5))),
+                              nps.mag, arr(5, dtype=int),
+                              dtype             = np.float32,
+                              out_inplace_dtype = np.float32)
+
     # input is a 1D array of integers, output dtype=int. Output should be an
     # integer scalar
     output = np.empty((), dtype=int)
@@ -1062,6 +1083,16 @@ def test_mag():
                                      nps.norm2(np.arange(5,10))))),
                   output)
 
+    # Make sure overflows can be handled by specifying a dtype
+    r = np.array((-206, 10), dtype=np.int16)
+    confirm_equal(206.*206. + 10.*10.,
+                  nps.norm2(r, dtype=float),
+                  msg = "norm2 can handle overflows with a dtype")
+
+    confirm_equal(np.sqrt(206.*206. + 10.*10.),
+                  nps.mag(r, dtype=float),
+                  msg = "mag can handle overflows with a dtype")
+
 
 def test_outer():
     r'''Testing the broadcasted outer product'''
